@@ -25,8 +25,9 @@
 #include <string.h>
 #include <stdio.h>
 #include <sys/param.h>
+
 #include <glib.h>
-#include <gnome.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include "file-data.h"
 #include "file-list.h"
@@ -536,6 +537,7 @@ action_started (FRCommand *command,
 		FRAction   action,
 		FRArchive *archive)
 {
+	g_print ("FRArchive::action_started: %d\n", action);
 	g_signal_emit (G_OBJECT (archive), 
 		       fr_archive_signals[START],
 		       0,
@@ -719,8 +721,8 @@ fr_archive_load (FRArchive   *archive,
 
                         return FALSE;
 		}
-	
-	if (tmp_command != NULL) {
+
+	if (tmp_command != NULL) { 
 		g_signal_handlers_disconnect_by_data (tmp_command, archive);
 		g_object_unref (tmp_command);
 	}
